# -*- coding: utf-8 -*-

# Copyright 2020 Google LLC
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
#


from google.cloud.recaptchaenterprise_v1.services.recaptcha_enterprise_service.client import (
    RecaptchaEnterpriseServiceClient,
)
from google.cloud.recaptchaenterprise_v1.types.recaptchaenterprise import (
    AndroidKeySettings,
)
from google.cloud.recaptchaenterprise_v1.types.recaptchaenterprise import (
    AnnotateAssessmentRequest,
)
from google.cloud.recaptchaenterprise_v1.types.recaptchaenterprise import (
    AnnotateAssessmentResponse,
)
from google.cloud.recaptchaenterprise_v1.types.recaptchaenterprise import Assessment
from google.cloud.recaptchaenterprise_v1.types.recaptchaenterprise import (
    CreateAssessmentRequest,
)
from google.cloud.recaptchaenterprise_v1.types.recaptchaenterprise import (
    CreateKeyRequest,
)
from google.cloud.recaptchaenterprise_v1.types.recaptchaenterprise import (
    DeleteKeyRequest,
)
from google.cloud.recaptchaenterprise_v1.types.recaptchaenterprise import Event
from google.cloud.recaptchaenterprise_v1.types.recaptchaenterprise import GetKeyRequest
from google.cloud.recaptchaenterprise_v1.types.recaptchaenterprise import IOSKeySettings
from google.cloud.recaptchaenterprise_v1.types.recaptchaenterprise import Key
from google.cloud.recaptchaenterprise_v1.types.recaptchaenterprise import (
    ListKeysRequest,
)
from google.cloud.recaptchaenterprise_v1.types.recaptchaenterprise import (
    ListKeysResponse,
)
from google.cloud.recaptchaenterprise_v1.types.recaptchaenterprise import RiskAnalysis
from google.cloud.recaptchaenterprise_v1.types.recaptchaenterprise import (
    TokenProperties,
)
from google.cloud.recaptchaenterprise_v1.types.recaptchaenterprise import (
    UpdateKeyRequest,
)
from google.cloud.recaptchaenterprise_v1.types.recaptchaenterprise import WebKeySettings

__all__ = (
    "AndroidKeySettings",
    "AnnotateAssessmentRequest",
    "AnnotateAssessmentResponse",
    "Assessment",
    "CreateAssessmentRequest",
    "CreateKeyRequest",
    "DeleteKeyRequest",
    "Event",
    "GetKeyRequest",
    "IOSKeySettings",
    "Key",
    "ListKeysRequest",
    "ListKeysResponse",
    "RecaptchaEnterpriseServiceClient",
    "RiskAnalysis",
    "TokenProperties",
    "UpdateKeyRequest",
    "WebKeySettings",
)



# -*- coding: utf-8 -*-

# Copyright 2020 Google LLC
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
#

from collections import OrderedDict
from typing import Dict, Sequence, Tuple, Type, Union
import pkg_resources

import google.api_core.client_options as ClientOptions  # type: ignore
from google.api_core import exceptions  # type: ignore
from google.api_core import gapic_v1  # type: ignore
from google.api_core import retry as retries  # type: ignore
from google.auth import credentials  # type: ignore
from google.oauth2 import service_account  # type: ignore

from google.cloud.recaptchaenterprise_v1.services.recaptcha_enterprise_service import (
    pagers,
)
from google.cloud.recaptchaenterprise_v1.types import recaptchaenterprise
from google.protobuf import timestamp_pb2 as timestamp  # type: ignore

from .transports.base import RecaptchaEnterpriseServiceTransport
from .transports.grpc import RecaptchaEnterpriseServiceGrpcTransport


class RecaptchaEnterpriseServiceClientMeta(type):
    """Metaclass for the RecaptchaEnterpriseService client.

    This provides class-level methods for building and retrieving
    support objects (e.g. transport) without polluting the client instance
    objects.
    """

    _transport_registry = (
        OrderedDict()
    )  # type: Dict[str, Type[RecaptchaEnterpriseServiceTransport]]
    _transport_registry["grpc"] = RecaptchaEnterpriseServiceGrpcTransport

    def get_transport_class(
        cls, label: str = None
    ) -> Type[RecaptchaEnterpriseServiceTransport]:
        """Return an appropriate transport class.

        Args:
            label: The name of the desired transport. If none is
                provided, then the first transport in the registry is used.

        Returns:
            The transport class to use.
        """
        # If a specific transport is requested, return that one.
        if label:
            return cls._transport_registry[label]

        # No transport is requested; return the default (that is, the first one
        # in the dictionary).
        return next(iter(cls._transport_registry.values()))


class RecaptchaEnterpriseServiceClient(metaclass=RecaptchaEnterpriseServiceClientMeta):
    """Service to determine the likelihood an event is legitimate."""

    DEFAULT_OPTIONS = ClientOptions.ClientOptions(
        api_endpoint="recaptchaenterprise.googleapis.com"
    )

    @classmethod
    def from_service_account_file(cls, filename: str, *args, **kwargs):
        """Creates an instance of this client using the provided credentials
        file.

        Args:
            filename (str): The path to the service account private key json
                file.
            args: Additional arguments to pass to the constructor.
            kwargs: Additional arguments to pass to the constructor.

        Returns:
            {@api.name}: The constructed client.
        """
        credentials = service_account.Credentials.from_service_account_file(filename)
        kwargs["credentials"] = credentials
        return cls(*args, **kwargs)

    from_service_account_json = from_service_account_file

    @staticmethod
    def key_path(project: str, key: str) -> str:
        """Return a fully-qualified key string."""
        return "projects/{project}/keys/{key}".format(project=project, key=key)

    @staticmethod
    def assessment_path(project: str, assessment: str) -> str:
        """Return a fully-qualified assessment string."""
        return "projects/{project}/assessments/{assessment}".format(
            project=project, assessment=assessment
        )

    def __init__(
        self,
        *,
        credentials: credentials.Credentials = None,
        transport: Union[str, RecaptchaEnterpriseServiceTransport] = None,
        client_options: ClientOptions = DEFAULT_OPTIONS,
    ) -> None:
        """Instantiate the recaptcha enterprise service client.

        Args:
            credentials (Optional[google.auth.credentials.Credentials]): The
                authorization credentials to attach to requests. These
                credentials identify the application to the service; if none
                are specified, the client will attempt to ascertain the
                credentials from the environment.
            transport (Union[str, ~.RecaptchaEnterpriseServiceTransport]): The
                transport to use. If set to None, a transport is chosen
                automatically.
            client_options (ClientOptions): Custom options for the client.
        """
        if isinstance(client_options, dict):
            client_options = ClientOptions.from_dict(client_options)

        # Save or instantiate the transport.
        # Ordinarily, we provide the transport, but allowing a custom transport
        # instance provides an extensibility point for unusual situations.
        if isinstance(transport, RecaptchaEnterpriseServiceTransport):
            if credentials:
                raise ValueError(
                    "When providing a transport instance, "
                    "provide its credentials directly."
                )
            self._transport = transport
        else:
            Transport = type(self).get_transport_class(transport)
            self._transport = Transport(
                credentials=credentials,
                host=client_options.api_endpoint
                or "recaptchaenterprise.googleapis.com",
            )

    def create_assessment(
        self,
        request: recaptchaenterprise.CreateAssessmentRequest = None,
        *,
        parent: str = None,
        assessment: recaptchaenterprise.Assessment = None,
        retry: retries.Retry = gapic_v1.method.DEFAULT,
        timeout: float = None,
        metadata: Sequence[Tuple[str, str]] = (),
    ) -> recaptchaenterprise.Assessment:
        r"""Creates an Assessment of the likelihood an event is
        legitimate.

        Args:
            request (:class:`~.recaptchaenterprise.CreateAssessmentRequest`):
                The request object. The create assessment request
                message.
            parent (:class:`str`):
                Required. The name of the project in
                which the assessment will be created, in
                the format "projects/{project}".
                This corresponds to the ``parent`` field
                on the ``request`` instance; if ``request`` is provided, this
                should not be set.
            assessment (:class:`~.recaptchaenterprise.Assessment`):
                Required. The assessment details.
                This corresponds to the ``assessment`` field
                on the ``request`` instance; if ``request`` is provided, this
                should not be set.

            retry (google.api_core.retry.Retry): Designation of what errors, if any,
                should be retried.
            timeout (float): The timeout for this request.
            metadata (Sequence[Tuple[str, str]]): Strings which should be
                sent along with the request as metadata.

        Returns:
            ~.recaptchaenterprise.Assessment:
                A recaptcha assessment resource.
        """
        # Create or coerce a protobuf request object.
        # Sanity check: If we got a request object, we should *not* have
        # gotten any keyword arguments that map to the request.
        if request is not None and any([parent, assessment]):
            raise ValueError(
                "If the `request` argument is set, then none of "
                "the individual field arguments should be set."
            )

        request = recaptchaenterprise.CreateAssessmentRequest(request)

        # If we have keyword arguments corresponding to fields on the
        # request, apply these.

        if parent is not None:
            request.parent = parent
        if assessment is not None:
            request.assessment = assessment

        # Wrap the RPC method; this adds retry and timeout information,
        # and friendly error handling.
        rpc = gapic_v1.method.wrap_method(
            self._transport.create_assessment,
            default_timeout=None,
            client_info=_client_info,
        )

        # Send the request.
        response = rpc(request, retry=retry, timeout=timeout, metadata=metadata)

        # Done; return the response.
        return response

    def annotate_assessment(
        self,
        request: recaptchaenterprise.AnnotateAssessmentRequest = None,
        *,
        name: str = None,
        annotation: recaptchaenterprise.AnnotateAssessmentRequest.Annotation = None,
        retry: retries.Retry = gapic_v1.method.DEFAULT,
        timeout: float = None,
        metadata: Sequence[Tuple[str, str]] = (),
    ) -> recaptchaenterprise.AnnotateAssessmentResponse:
        r"""Annotates a previously created Assessment to provide
        additional information on whether the event turned out
        to be authentic or fradulent.

        Args:
            request (:class:`~.recaptchaenterprise.AnnotateAssessmentRequest`):
                The request object. The request message to annotate an
                Assessment.
            name (:class:`str`):
                Required. The resource name of the
                Assessment, in the format
                "projects/{project}/assessments/{assessment}".
                This corresponds to the ``name`` field
                on the ``request`` instance; if ``request`` is provided, this
                should not be set.
            annotation (:class:`~.recaptchaenterprise.AnnotateAssessmentRequest.Annotation`):
                Required. The annotation that will be
                assigned to the Event.
                This corresponds to the ``annotation`` field
                on the ``request`` instance; if ``request`` is provided, this
                should not be set.

            retry (google.api_core.retry.Retry): Designation of what errors, if any,
                should be retried.
            timeout (float): The timeout for this request.
            metadata (Sequence[Tuple[str, str]]): Strings which should be
                sent along with the request as metadata.

        Returns:
            ~.recaptchaenterprise.AnnotateAssessmentResponse:
                Empty response for
                AnnotateAssessment.

        """
        # Create or coerce a protobuf request object.
        # Sanity check: If we got a request object, we should *not* have
        # gotten any keyword arguments that map to the request.
        if request is not None and any([name, annotation]):
            raise ValueError(
                "If the `request` argument is set, then none of "
                "the individual field arguments should be set."
            )

        request = recaptchaenterprise.AnnotateAssessmentRequest(request)

        # If we have keyword arguments corresponding to fields on the
        # request, apply these.

        if name is not None:
            request.name = name
        if annotation is not None:
            request.annotation = annotation

        # Wrap the RPC method; this adds retry and timeout information,
        # and friendly error handling.
        rpc = gapic_v1.method.wrap_method(
            self._transport.annotate_assessment,
            default_timeout=None,
            client_info=_client_info,
        )

        # Send the request.
        response = rpc(request, retry=retry, timeout=timeout, metadata=metadata)

        # Done; return the response.
        return response

    def create_key(
        self,
        request: recaptchaenterprise.CreateKeyRequest = None,
        *,
        retry: retries.Retry = gapic_v1.method.DEFAULT,
        timeout: float = None,
        metadata: Sequence[Tuple[str, str]] = (),
    ) -> recaptchaenterprise.Key:
        r"""Creates a new reCAPTCHA Enterprise key.

        Args:
            request (:class:`~.recaptchaenterprise.CreateKeyRequest`):
                The request object. The create key request message.

            retry (google.api_core.retry.Retry): Designation of what errors, if any,
                should be retried.
            timeout (float): The timeout for this request.
            metadata (Sequence[Tuple[str, str]]): Strings which should be
                sent along with the request as metadata.

        Returns:
            ~.recaptchaenterprise.Key:
                A key used to identify and configure
                applications (web and/or mobile) that
                use reCAPTCHA Enterprise.

        """
        # Create or coerce a protobuf request object.

        request = recaptchaenterprise.CreateKeyRequest(request)

        # Wrap the RPC method; this adds retry and timeout information,
        # and friendly error handling.
        rpc = gapic_v1.method.wrap_method(
            self._transport.create_key, default_timeout=None, client_info=_client_info
        )

        # Send the request.
        response = rpc(request, retry=retry, timeout=timeout, metadata=metadata)

        # Done; return the response.
        return response

    def list_keys(
        self,
        request: recaptchaenterprise.ListKeysRequest = None,
        *,
        retry: retries.Retry = gapic_v1.method.DEFAULT,
        timeout: float = None,
        metadata: Sequence[Tuple[str, str]] = (),
    ) -> pagers.ListKeysPager:
        r"""Returns the list of all keys that belong to a
        project.

        Args:
            request (:class:`~.recaptchaenterprise.ListKeysRequest`):
                The request object. The list keys request message.

            retry (google.api_core.retry.Retry): Designation of what errors, if any,
                should be retried.
            timeout (float): The timeout for this request.
            metadata (Sequence[Tuple[str, str]]): Strings which should be
                sent along with the request as metadata.

        Returns:
            ~.pagers.ListKeysPager:
                Response to request to list keys in a
                project.
                Iterating over this object will yield
                results and resolve additional pages
                automatically.

        """
        # Create or coerce a protobuf request object.

        request = recaptchaenterprise.ListKeysRequest(request)

        # Wrap the RPC method; this adds retry and timeout information,
        # and friendly error handling.
        rpc = gapic_v1.method.wrap_method(
            self._transport.list_keys, default_timeout=None, client_info=_client_info
        )

        # Certain fields should be provided within the metadata header;
        # add these here.
        metadata = tuple(metadata) + (
            gapic_v1.routing_header.to_grpc_metadata((("parent", request.parent),)),
        )

        # Send the request.
        response = rpc(request, retry=retry, timeout=timeout, metadata=metadata)

        # This method is paged; wrap the response in a pager, which provides
        # an `__iter__` convenience method.
        response = pagers.ListKeysPager(method=rpc, request=request, response=response)

        # Done; return the response.
        return response

    def get_key(
        self,
        request: recaptchaenterprise.GetKeyRequest = None,
        *,
        retry: retries.Retry = gapic_v1.method.DEFAULT,
        timeout: float = None,
        metadata: Sequence[Tuple[str, str]] = (),
    ) -> recaptchaenterprise.Key:
        r"""Returns the specified key.

        Args:
            request (:class:`~.recaptchaenterprise.GetKeyRequest`):
                The request object. The get key request message.

            retry (google.api_core.retry.Retry): Designation of what errors, if any,
                should be retried.
            timeout (float): The timeout for this request.
            metadata (Sequence[Tuple[str, str]]): Strings which should be
                sent along with the request as metadata.

        Returns:
            ~.recaptchaenterprise.Key:
                A key used to identify and configure
                applications (web and/or mobile) that
                use reCAPTCHA Enterprise.

        """
        # Create or coerce a protobuf request object.

        request = recaptchaenterprise.GetKeyRequest(request)

        # Wrap the RPC method; this adds retry and timeout information,
        # and friendly error handling.
        rpc = gapic_v1.method.wrap_method(
            self._transport.get_key, default_timeout=None, client_info=_client_info
        )

        # Certain fields should be provided within the metadata header;
        # add these here.
        metadata = tuple(metadata) + (
            gapic_v1.routing_header.to_grpc_metadata((("name", request.name),)),
        )

        # Send the request.
        response = rpc(request, retry=retry, timeout=timeout, metadata=metadata)

        # Done; return the response.
        return response

    def update_key(
        self,
        request: recaptchaenterprise.UpdateKeyRequest = None,
        *,
        retry: retries.Retry = gapic_v1.method.DEFAULT,
        timeout: float = None,
        metadata: Sequence[Tuple[str, str]] = (),
    ) -> recaptchaenterprise.Key:
        r"""Updates the specified key.

        Args:
            request (:class:`~.recaptchaenterprise.UpdateKeyRequest`):
                The request object. The update key request message.

            retry (google.api_core.retry.Retry): Designation of what errors, if any,
                should be retried.
            timeout (float): The timeout for this request.
            metadata (Sequence[Tuple[str, str]]): Strings which should be
                sent along with the request as metadata.

        Returns:
            ~.recaptchaenterprise.Key:
                A key used to identify and configure
                applications (web and/or mobile) that
                use reCAPTCHA Enterprise.

        """
        # Create or coerce a protobuf request object.

        request = recaptchaenterprise.UpdateKeyRequest(request)

        # Wrap the RPC method; this adds retry and timeout information,
        # and friendly error handling.
        rpc = gapic_v1.method.wrap_method(
            self._transport.update_key, default_timeout=None, client_info=_client_info
        )

        # Send the request.
        response = rpc(request, retry=retry, timeout=timeout, metadata=metadata)

        # Done; return the response.
        return response

    def delete_key(
        self,
        request: recaptchaenterprise.DeleteKeyRequest = None,
        *,
        retry: retries.Retry = gapic_v1.method.DEFAULT,
        timeout: float = None,
        metadata: Sequence[Tuple[str, str]] = (),
    ) -> None:
        r"""Deletes the specified key.

        Args:
            request (:class:`~.recaptchaenterprise.DeleteKeyRequest`):
                The request object. The delete key request message.

            retry (google.api_core.retry.Retry): Designation of what errors, if any,
                should be retried.
            timeout (float): The timeout for this request.
            metadata (Sequence[Tuple[str, str]]): Strings which should be
                sent along with the request as metadata.
        """
        # Create or coerce a protobuf request object.

        request = recaptchaenterprise.DeleteKeyRequest(request)

        # Wrap the RPC method; this adds retry and timeout information,
        # and friendly error handling.
        rpc = gapic_v1.method.wrap_method(
            self._transport.delete_key, default_timeout=None, client_info=_client_info
        )

        # Send the request.
        rpc(request, retry=retry, timeout=timeout, metadata=metadata)


try:
    _client_info = gapic_v1.client_info.ClientInfo(
        gapic_version=pkg_resources.get_distribution(
            "google-cloud-recaptcha-enterprise"
        ).version
    )
except pkg_resources.DistributionNotFound:
    _client_info = gapic_v1.client_info.ClientInfo()


__all__ = ("RecaptchaEnterpriseServiceClient",)



# -*- coding: utf-8 -*-

# Copyright 2020 Google LLC
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
#

from typing import Callable, Dict

from google.api_core import grpc_helpers  # type: ignore
from google.auth import credentials  # type: ignore

import grpc  # type: ignore

from google.cloud.recaptchaenterprise_v1.types import recaptchaenterprise
from google.protobuf import empty_pb2 as empty  # type: ignore

from .base import RecaptchaEnterpriseServiceTransport


class RecaptchaEnterpriseServiceGrpcTransport(RecaptchaEnterpriseServiceTransport):
    """gRPC backend transport for RecaptchaEnterpriseService.

    Service to determine the likelihood an event is legitimate.

    This class defines the same methods as the primary client, so the
    primary client can load the underlying transport implementation
    and call it.

    It sends protocol buffers over the wire using gRPC (which is built on
    top of HTTP/2); the ``grpcio`` package must be installed.
    """

    def __init__(
        self,
        *,
        host: str = "recaptchaenterprise.googleapis.com",
        credentials: credentials.Credentials = None,
        channel: grpc.Channel = None
    ) -> None:
        """Instantiate the transport.

        Args:
            host (Optional[str]): The hostname to connect to.
            credentials (Optional[google.auth.credentials.Credentials]): The
                authorization credentials to attach to requests. These
                credentials identify the application to the service; if none
                are specified, the client will attempt to ascertain the
                credentials from the environment.
                This argument is ignored if ``channel`` is provided.
            channel (Optional[grpc.Channel]): A ``Channel`` instance through
                which to make calls.
        """
        # Sanity check: Ensure that channel and credentials are not both
        # provided.
        if channel:
            credentials = False

        # Run the base constructor.
        super().__init__(host=host, credentials=credentials)
        self._stubs = {}  # type: Dict[str, Callable]

        # If a channel was explicitly provided, set it.
        if channel:
            self._grpc_channel = channel

    @classmethod
    def create_channel(
        cls,
        host: str = "recaptchaenterprise.googleapis.com",
        credentials: credentials.Credentials = None,
        **kwargs
    ) -> grpc.Channel:
        """Create and return a gRPC channel object.
        Args:
            address (Optionsl[str]): The host for the channel to use.
            credentials (Optional[~.Credentials]): The
                authorization credentials to attach to requests. These
                credentials identify this application to the service. If
                none are specified, the client will attempt to ascertain
                the credentials from the environment.
            kwargs (Optional[dict]): Keyword arguments, which are passed to the
                channel creation.
        Returns:
            grpc.Channel: A gRPC channel object.
        """
        return grpc_helpers.create_channel(
            host, credentials=credentials, scopes=cls.AUTH_SCOPES, **kwargs
        )

    @property
    def grpc_channel(self) -> grpc.Channel:
        """Create the channel designed to connect to this service.

        This property caches on the instance; repeated calls return
        the same channel.
        """
        # Sanity check: Only create a new channel if we do not already
        # have one.
        if not hasattr(self, "_grpc_channel"):
            self._grpc_channel = self.create_channel(
                self._host, credentials=self._credentials
            )

        # Return the channel from cache.
        return self._grpc_channel

    @property
    def create_assessment(
        self
    ) -> Callable[
        [recaptchaenterprise.CreateAssessmentRequest], recaptchaenterprise.Assessment
    ]:
        r"""Return a callable for the create assessment method over gRPC.

        Creates an Assessment of the likelihood an event is
        legitimate.

        Returns:
            Callable[[~.CreateAssessmentRequest],
                    ~.Assessment]:
                A function that, when called, will call the underlying RPC
                on the server.
        """
        # Generate a "stub function" on-the-fly which will actually make
        # the request.
        # gRPC handles serialization and deserialization, so we just need
        # to pass in the functions for each.
        if "create_assessment" not in self._stubs:
            self._stubs["create_assessment"] = self.grpc_channel.unary_unary(
                "/google.cloud.recaptchaenterprise.v1.RecaptchaEnterpriseService/CreateAssessment",
                request_serializer=recaptchaenterprise.CreateAssessmentRequest.serialize,
                response_deserializer=recaptchaenterprise.Assessment.deserialize,
            )
        return self._stubs["create_assessment"]

    @property
    def annotate_assessment(
        self
    ) -> Callable[
        [recaptchaenterprise.AnnotateAssessmentRequest],
        recaptchaenterprise.AnnotateAssessmentResponse,
    ]:
        r"""Return a callable for the annotate assessment method over gRPC.

        Annotates a previously created Assessment to provide
        additional information on whether the event turned out
        to be authentic or fradulent.

        Returns:
            Callable[[~.AnnotateAssessmentRequest],
                    ~.AnnotateAssessmentResponse]:
                A function that, when called, will call the underlying RPC
                on the server.
        """
        # Generate a "stub function" on-the-fly which will actually make
        # the request.
        # gRPC handles serialization and deserialization, so we just need
        # to pass in the functions for each.
        if "annotate_assessment" not in self._stubs:
            self._stubs["annotate_assessment"] = self.grpc_channel.unary_unary(
                "/google.cloud.recaptchaenterprise.v1.RecaptchaEnterpriseService/AnnotateAssessment",
                request_serializer=recaptchaenterprise.AnnotateAssessmentRequest.serialize,
                response_deserializer=recaptchaenterprise.AnnotateAssessmentResponse.deserialize,
            )
        return self._stubs["annotate_assessment"]

    @property
    def create_key(
        self
    ) -> Callable[[recaptchaenterprise.CreateKeyRequest], recaptchaenterprise.Key]:
        r"""Return a callable for the create key method over gRPC.

        Creates a new reCAPTCHA Enterprise key.

        Returns:
            Callable[[~.CreateKeyRequest],
                    ~.Key]:
                A function that, when called, will call the underlying RPC
                on the server.
        """
        # Generate a "stub function" on-the-fly which will actually make
        # the request.
        # gRPC handles serialization and deserialization, so we just need
        # to pass in the functions for each.
        if "create_key" not in self._stubs:
            self._stubs["create_key"] = self.grpc_channel.unary_unary(
                "/google.cloud.recaptchaenterprise.v1.RecaptchaEnterpriseService/CreateKey",
                request_serializer=recaptchaenterprise.CreateKeyRequest.serialize,
                response_deserializer=recaptchaenterprise.Key.deserialize,
            )
        return self._stubs["create_key"]

    @property
    def list_keys(
        self
    ) -> Callable[
        [recaptchaenterprise.ListKeysRequest], recaptchaenterprise.ListKeysResponse
    ]:
        r"""Return a callable for the list keys method over gRPC.

        Returns the list of all keys that belong to a
        project.

        Returns:
            Callable[[~.ListKeysRequest],
                    ~.ListKeysResponse]:
                A function that, when called, will call the underlying RPC
                on the server.
        """
        # Generate a "stub function" on-the-fly which will actually make
        # the request.
        # gRPC handles serialization and deserialization, so we just need
        # to pass in the functions for each.
        if "list_keys" not in self._stubs:
            self._stubs["list_keys"] = self.grpc_channel.unary_unary(
                "/google.cloud.recaptchaenterprise.v1.RecaptchaEnterpriseService/ListKeys",
                request_serializer=recaptchaenterprise.ListKeysRequest.serialize,
                response_deserializer=recaptchaenterprise.ListKeysResponse.deserialize,
            )
        return self._stubs["list_keys"]

    @property
    def get_key(
        self
    ) -> Callable[[recaptchaenterprise.GetKeyRequest], recaptchaenterprise.Key]:
        r"""Return a callable for the get key method over gRPC.

        Returns the specified key.

        Returns:
            Callable[[~.GetKeyRequest],
                    ~.Key]:
                A function that, when called, will call the underlying RPC
                on the server.
        """
        # Generate a "stub function" on-the-fly which will actually make
        # the request.
        # gRPC handles serialization and deserialization, so we just need
        # to pass in the functions for each.
        if "get_key" not in self._stubs:
            self._stubs["get_key"] = self.grpc_channel.unary_unary(
                "/google.cloud.recaptchaenterprise.v1.RecaptchaEnterpriseService/GetKey",
                request_serializer=recaptchaenterprise.GetKeyRequest.serialize,
                response_deserializer=recaptchaenterprise.Key.deserialize,
            )
        return self._stubs["get_key"]

    @property
    def update_key(
        self
    ) -> Callable[[recaptchaenterprise.UpdateKeyRequest], recaptchaenterprise.Key]:
        r"""Return a callable for the update key method over gRPC.

        Updates the specified key.

        Returns:
            Callable[[~.UpdateKeyRequest],
                    ~.Key]:
                A function that, when called, will call the underlying RPC
                on the server.
        """
        # Generate a "stub function" on-the-fly which will actually make
        # the request.
        # gRPC handles serialization and deserialization, so we just need
        # to pass in the functions for each.
        if "update_key" not in self._stubs:
            self._stubs["update_key"] = self.grpc_channel.unary_unary(
                "/google.cloud.recaptchaenterprise.v1.RecaptchaEnterpriseService/UpdateKey",
                request_serializer=recaptchaenterprise.UpdateKeyRequest.serialize,
                response_deserializer=recaptchaenterprise.Key.deserialize,
            )
        return self._stubs["update_key"]

    @property
    def delete_key(
        self
    ) -> Callable[[recaptchaenterprise.DeleteKeyRequest], empty.Empty]:
        r"""Return a callable for the delete key method over gRPC.

        Deletes the specified key.

        Returns:
            Callable[[~.DeleteKeyRequest],
                    ~.Empty]:
                A function that, when called, will call the underlying RPC
                on the server.
        """
        # Generate a "stub function" on-the-fly which will actually make
        # the request.
        # gRPC handles serialization and deserialization, so we just need
        # to pass in the functions for each.
        if "delete_key" not in self._stubs:
            self._stubs["delete_key"] = self.grpc_channel.unary_unary(
                "/google.cloud.recaptchaenterprise.v1.RecaptchaEnterpriseService/DeleteKey",
                request_serializer=recaptchaenterprise.DeleteKeyRequest.serialize,
                response_deserializer=empty.Empty.FromString,
            )
        return self._stubs["delete_key"]


__all__ = ("RecaptchaEnterpriseServiceGrpcTransport",)



# -*- coding: utf-8 -*-

# Copyright 2020 Google LLC
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
#

import abc
import typing

from google import auth
from google.auth import credentials  # type: ignore

from google.cloud.recaptchaenterprise_v1.types import recaptchaenterprise
from google.protobuf import empty_pb2 as empty  # type: ignore


class RecaptchaEnterpriseServiceTransport(metaclass=abc.ABCMeta):
    """Abstract transport class for RecaptchaEnterpriseService."""

    AUTH_SCOPES = ("https://www.googleapis.com/auth/cloud-platform",)

    def __init__(
        self,
        *,
        host: str = "recaptchaenterprise.googleapis.com",
        credentials: credentials.Credentials = None,
    ) -> None:
        """Instantiate the transport.

        Args:
            host (Optional[str]): The hostname to connect to.
            credentials (Optional[google.auth.credentials.Credentials]): The
                authorization credentials to attach to requests. These
                credentials identify the application to the service; if none
                are specified, the client will attempt to ascertain the
                credentials from the environment.
        """
        # Save the hostname. Default to port 443 (HTTPS) if none is specified.
        if ":" not in host:
            host += ":443"
        self._host = host

        # If no credentials are provided, then determine the appropriate
        # defaults.
        if credentials is None:
            credentials, _ = auth.default(scopes=self.AUTH_SCOPES)

        # Save the credentials.
        self._credentials = credentials

    @property
    def create_assessment(
        self
    ) -> typing.Callable[
        [recaptchaenterprise.CreateAssessmentRequest], recaptchaenterprise.Assessment
    ]:
        raise NotImplementedError

    @property
    def annotate_assessment(
        self
    ) -> typing.Callable[
        [recaptchaenterprise.AnnotateAssessmentRequest],
        recaptchaenterprise.AnnotateAssessmentResponse,
    ]:
        raise NotImplementedError

    @property
    def create_key(
        self
    ) -> typing.Callable[
        [recaptchaenterprise.CreateKeyRequest], recaptchaenterprise.Key
    ]:
        raise NotImplementedError

    @property
    def list_keys(
        self
    ) -> typing.Callable[
        [recaptchaenterprise.ListKeysRequest], recaptchaenterprise.ListKeysResponse
    ]:
        raise NotImplementedError

    @property
    def get_key(
        self
    ) -> typing.Callable[[recaptchaenterprise.GetKeyRequest], recaptchaenterprise.Key]:
        raise NotImplementedError

    @property
    def update_key(
        self
    ) -> typing.Callable[
        [recaptchaenterprise.UpdateKeyRequest], recaptchaenterprise.Key
    ]:
        raise NotImplementedError

    @property
    def delete_key(
        self
    ) -> typing.Callable[[recaptchaenterprise.DeleteKeyRequest], empty.Empty]:
        raise NotImplementedError


__all__ = ("RecaptchaEnterpriseServiceTransport",)



# -*- coding: utf-8 -*-

# Copyright 2020 Google LLC
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
#

from .recaptchaenterprise import (
    CreateAssessmentRequest,
    AnnotateAssessmentRequest,
    AnnotateAssessmentResponse,
    Assessment,
    Event,
    RiskAnalysis,
    TokenProperties,
    CreateKeyRequest,
    ListKeysRequest,
    ListKeysResponse,
    GetKeyRequest,
    UpdateKeyRequest,
    DeleteKeyRequest,
    Key,
    WebKeySettings,
    AndroidKeySettings,
    IOSKeySettings,
)


__all__ = (
    "CreateAssessmentRequest",
    "AnnotateAssessmentRequest",
    "AnnotateAssessmentResponse",
    "Assessment",
    "Event",
    "RiskAnalysis",
    "TokenProperties",
    "CreateKeyRequest",
    "ListKeysRequest",
    "ListKeysResponse",
    "GetKeyRequest",
    "UpdateKeyRequest",
    "DeleteKeyRequest",
    "Key",
    "WebKeySettings",
    "AndroidKeySettings",
    "IOSKeySettings",
)



# -*- coding: utf-8 -*-

# Copyright 2020 Google LLC
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
#

import proto  # type: ignore


from google.protobuf import field_mask_pb2 as field_mask  # type: ignore
from google.protobuf import timestamp_pb2 as timestamp  # type: ignore


__protobuf__ = proto.module(
    package="google.cloud.recaptchaenterprise.v1",
    manifest={
        "CreateAssessmentRequest",
        "AnnotateAssessmentRequest",
        "AnnotateAssessmentResponse",
        "Assessment",
        "Event",
        "RiskAnalysis",
        "TokenProperties",
        "CreateKeyRequest",
        "ListKeysRequest",
        "ListKeysResponse",
        "GetKeyRequest",
        "UpdateKeyRequest",
        "DeleteKeyRequest",
        "Key",
        "WebKeySettings",
        "AndroidKeySettings",
        "IOSKeySettings",
    },
)


class CreateAssessmentRequest(proto.Message):
    r"""The create assessment request message.

    Attributes:
        parent (str):
            Required. The name of the project in which
            the assessment will be created, in the format
            "projects/{project}".
        assessment (~.recaptchaenterprise.Assessment):
            Required. The assessment details.
    """

    parent = proto.Field(proto.STRING, number=1)
    assessment = proto.Field(proto.MESSAGE, number=2, message="Assessment")


class AnnotateAssessmentRequest(proto.Message):
    r"""The request message to annotate an Assessment.

    Attributes:
        name (str):
            Required. The resource name of the
            Assessment, in the format
            "projects/{project}/assessments/{assessment}".
        annotation (~.recaptchaenterprise.AnnotateAssessmentRequest.Annotation):
            Required. The annotation that will be
            assigned to the Event.
    """

    class Annotation(proto.Enum):
        r"""Enum that reprensents the types of annotations."""
        ANNOTATION_UNSPECIFIED = 0
        LEGITIMATE = 1
        FRAUDULENT = 2
        PASSWORD_CORRECT = 3
        PASSWORD_INCORRECT = 4

    name = proto.Field(proto.STRING, number=1)
    annotation = proto.Field(proto.ENUM, number=2, enum=Annotation)


class AnnotateAssessmentResponse(proto.Message):
    r"""Empty response for AnnotateAssessment."""


class Assessment(proto.Message):
    r"""A recaptcha assessment resource.

    Attributes:
        name (str):
            Output only. The resource name for the
            Assessment in the format
            "projects/{project}/assessments/{assessment}".
        event (~.recaptchaenterprise.Event):
            The event being assessed.
        risk_analysis (~.recaptchaenterprise.RiskAnalysis):
            Output only. The risk analysis result for the
            event being assessed.
        token_properties (~.recaptchaenterprise.TokenProperties):
            Output only. Properties of the provided event
            token.
    """

    name = proto.Field(proto.STRING, number=1)
    event = proto.Field(proto.MESSAGE, number=2, message="Event")
    risk_analysis = proto.Field(proto.MESSAGE, number=3, message="RiskAnalysis")
    token_properties = proto.Field(proto.MESSAGE, number=4, message="TokenProperties")


class Event(proto.Message):
    r"""

    Attributes:
        token (str):
            Optional. The user response token provided by
            the reCAPTCHA client-side integration on your
            site.
        site_key (str):
            Optional. The site key that was used to
            invoke reCAPTCHA on your site and generate the
            token.
        user_agent (str):
            Optional. The user agent present in the
            request from the user's device related to this
            event.
        user_ip_address (str):
            Optional. The IP address in the request from
            the user's device related to this event.
        expected_action (str):
            Optional. The expected action for this type
            of event. This should be the same action
            provided at token generation time on client-side
            platforms already integrated with recaptcha
            enterprise.
    """

    token = proto.Field(proto.STRING, number=1)
    site_key = proto.Field(proto.STRING, number=2)
    user_agent = proto.Field(proto.STRING, number=3)
    user_ip_address = proto.Field(proto.STRING, number=4)
    expected_action = proto.Field(proto.STRING, number=5)


class RiskAnalysis(proto.Message):
    r"""Risk analysis result for an event.

    Attributes:
        score (float):
            Legitimate event score from 0.0 to 1.0.
            (1.0 means very likely legitimate traffic while
            0.0 means very likely non-legitimate traffic).
        reasons (Sequence[~.recaptchaenterprise.RiskAnalysis.ClassificationReason]):
            Reasons contributing to the risk analysis
            verdict.
    """

    class ClassificationReason(proto.Enum):
        r"""LINT.IfChange(classification_reason) Reasons contributing to the
        risk analysis verdict.
        """
        CLASSIFICATION_REASON_UNSPECIFIED = 0
        AUTOMATION = 1
        UNEXPECTED_ENVIRONMENT = 2
        TOO_MUCH_TRAFFIC = 3
        UNEXPECTED_USAGE_PATTERNS = 4
        LOW_CONFIDENCE_SCORE = 5

    score = proto.Field(proto.FLOAT, number=1)
    reasons = proto.RepeatedField(proto.ENUM, number=2, enum=ClassificationReason)


class TokenProperties(proto.Message):
    r"""

    Attributes:
        valid (bool):
            Whether the provided user response token is valid. When
            valid = false, the reason could be specified in
            invalid_reason or it could also be due to a user failing to
            solve a challenge or a sitekey mismatch (i.e the sitekey
            used to generate the token was different than the one
            specified in the assessment).
        invalid_reason (~.recaptchaenterprise.TokenProperties.InvalidReason):
            Reason associated with the response when
            valid = false.
        create_time (~.timestamp.Timestamp):
            The timestamp corresponding to the generation
            of the token.
        hostname (str):
            The hostname of the page on which the token
            was generated.
        action (str):
            Action name provided at token generation.
    """

    class InvalidReason(proto.Enum):
        r"""LINT.IfChange
        Enum that represents the types of invalid token reasons.
        """
        INVALID_REASON_UNSPECIFIED = 0
        UNKNOWN_INVALID_REASON = 1
        MALFORMED = 2
        EXPIRED = 3
        DUPE = 4
        MISSING = 5

    valid = proto.Field(proto.BOOL, number=1)
    invalid_reason = proto.Field(proto.ENUM, number=2, enum=InvalidReason)
    create_time = proto.Field(proto.MESSAGE, number=3, message=timestamp.Timestamp)
    hostname = proto.Field(proto.STRING, number=4)
    action = proto.Field(proto.STRING, number=5)


class CreateKeyRequest(proto.Message):
    r"""The create key request message.

    Attributes:
        parent (str):
            Required. The name of the project in which
            the key will be created, in the format
            "projects/{project}".
        key (~.recaptchaenterprise.Key):
            Required. Information to create a reCAPTCHA
            Enterprise key.
    """

    parent = proto.Field(proto.STRING, number=1)
    key = proto.Field(proto.MESSAGE, number=2, message="Key")


class ListKeysRequest(proto.Message):
    r"""The list keys request message.

    Attributes:
        parent (str):
            Required. The name of the project that
            contains the keys that will be listed, in the
            format "projects/{project}".
        page_size (int):
            Optional. The maximum number of keys to
            return. Default is 10. Max limit is 1000.
        page_token (str):
            Optional. The next_page_token value returned from a
            previous. ListKeysRequest, if any.
    """

    parent = proto.Field(proto.STRING, number=1)
    page_size = proto.Field(proto.INT32, number=2)
    page_token = proto.Field(proto.STRING, number=3)


class ListKeysResponse(proto.Message):
    r"""Response to request to list keys in a project.

    Attributes:
        keys (Sequence[~.recaptchaenterprise.Key]):
            Key details.
        next_page_token (str):
            Token to retrieve the next page of results.
            It is set to empty if no keys remain in results.
    """

    @property
    def raw_page(self):
        return self

    keys = proto.RepeatedField(proto.MESSAGE, number=1, message="Key")
    next_page_token = proto.Field(proto.STRING, number=2)


class GetKeyRequest(proto.Message):
    r"""The get key request message.

    Attributes:
        name (str):
            Required. The name of the requested key, in
            the format "projects/{project}/keys/{key}".
    """

    name = proto.Field(proto.STRING, number=1)


class UpdateKeyRequest(proto.Message):
    r"""The update key request message.

    Attributes:
        key (~.recaptchaenterprise.Key):
            Required. The key to update.
        update_mask (~.field_mask.FieldMask):
            Optional. The mask to control which field of
            the key get updated. If the mask is not present,
            all fields will be updated.
    """

    key = proto.Field(proto.MESSAGE, number=1, message="Key")
    update_mask = proto.Field(proto.MESSAGE, number=2, message=field_mask.FieldMask)


class DeleteKeyRequest(proto.Message):
    r"""The delete key request message.

    Attributes:
        name (str):
            Required. The name of the key to be deleted,
            in the format "projects/{project}/keys/{key}".
    """

    name = proto.Field(proto.STRING, number=1)


class Key(proto.Message):
    r"""A key used to identify and configure applications (web and/or
    mobile) that use reCAPTCHA Enterprise.

    Attributes:
        name (str):
            The resource name for the Key in the format
            "projects/{project}/keys/{key}".
        display_name (str):
            Human-readable display name of this key.
            Modifiable by user.
        web_settings (~.recaptchaenterprise.WebKeySettings):
            Settings for keys that can be used by
            websites.
        android_settings (~.recaptchaenterprise.AndroidKeySettings):
            Settings for keys that can be used by Android
            apps.
        ios_settings (~.recaptchaenterprise.IOSKeySettings):
            Settings for keys that can be used by iOS
            apps.
        labels (Sequence[~.recaptchaenterprise.Key.LabelsEntry]):
            Optional. See <a
            href="https://cloud.google.com/recaptcha-
            enterprise/docs/labels"> Creating and managing
            labels</a>.
        create_time (~.timestamp.Timestamp):
            The timestamp corresponding to the creation
            of this Key.
    """

    name = proto.Field(proto.STRING, number=1)
    display_name = proto.Field(proto.STRING, number=2)
    web_settings = proto.Field(proto.MESSAGE, number=3, message="WebKeySettings")
    android_settings = proto.Field(
        proto.MESSAGE, number=4, message="AndroidKeySettings"
    )
    ios_settings = proto.Field(proto.MESSAGE, number=5, message="IOSKeySettings")
    labels = proto.MapField(proto.STRING, proto.STRING, number=6)
    create_time = proto.Field(proto.MESSAGE, number=7, message=timestamp.Timestamp)


class WebKeySettings(proto.Message):
    r"""Settings specific to keys that can be used by websites.

    Attributes:
        allow_all_domains (bool):
            If set to true, it means allowed_domains will not be
            enforced.
        allowed_domains (Sequence[str]):
            Domains or subdomains of websites allowed to
            use the key. All subdomains of an allowed domain
            are automatically allowed. A valid domain
            requires a host and must not include any path,
            port, query or fragment. Examples: 'example.com'
            or 'subdomain.example.com'
        allow_amp_traffic (bool):
            Required. Whether this key can be used on AMP
            (Accelerated Mobile Pages) websites.
        integration_type (~.recaptchaenterprise.WebKeySettings.IntegrationType):
            Required. Describes how this key is
            integrated with the website.
        challenge_security_preference (~.recaptchaenterprise.WebKeySettings.ChallengeSecurityPreference):
            Settings for the frequency and difficulty at
            which this key triggers captcha challenges. This
            should only be specified for IntegrationTypes
            CHECKBOX and INVISIBLE.
    """

    class IntegrationType(proto.Enum):
        r"""Enum that represents the integration types for web keys."""
        INTEGRATION_TYPE_UNSPECIFIED = 0
        SCORE = 1
        CHECKBOX = 2
        INVISIBLE = 3

    class ChallengeSecurityPreference(proto.Enum):
        r"""Enum that represents the possible challenge frequency and
        difficulty configurations for a web key.
        """
        CHALLENGE_SECURITY_PREFERENCE_UNSPECIFIED = 0
        USABILITY = 1
        BALANCE = 2
        SECURITY = 3

    allow_all_domains = proto.Field(proto.BOOL, number=3)
    allowed_domains = proto.RepeatedField(proto.STRING, number=1)
    allow_amp_traffic = proto.Field(proto.BOOL, number=2)
    integration_type = proto.Field(proto.ENUM, number=4, enum=IntegrationType)
    challenge_security_preference = proto.Field(
        proto.ENUM, number=5, enum=ChallengeSecurityPreference
    )


class AndroidKeySettings(proto.Message):
    r"""Settings specific to keys that can be used by Android apps.

    Attributes:
        allowed_package_names (Sequence[str]):
            Android package names of apps allowed to use
            the key. Example: 'com.companyname.appname'
    """

    allowed_package_names = proto.RepeatedField(proto.STRING, number=1)


class IOSKeySettings(proto.Message):
    r"""Settings specific to keys that can be used by iOS apps.

    Attributes:
        allowed_bundle_ids (Sequence[str]):
            iOS bundle ids of apps allowed to use the
            key. Example:
            'com.companyname.productname.appname'
    """

    allowed_bundle_ids = proto.RepeatedField(proto.STRING, number=1)


__all__ = tuple(sorted(__protobuf__.manifest))



# -*- coding: utf-8 -*-

# Copyright 2020 Google LLC
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
#


from .services.recaptcha_enterprise_service import RecaptchaEnterpriseServiceClient
from .types.recaptchaenterprise import AndroidKeySettings
from .types.recaptchaenterprise import AnnotateAssessmentRequest
from .types.recaptchaenterprise import AnnotateAssessmentResponse
from .types.recaptchaenterprise import Assessment
from .types.recaptchaenterprise import CreateAssessmentRequest
from .types.recaptchaenterprise import CreateKeyRequest
from .types.recaptchaenterprise import DeleteKeyRequest
from .types.recaptchaenterprise import Event
from .types.recaptchaenterprise import GetKeyRequest
from .types.recaptchaenterprise import IOSKeySettings
from .types.recaptchaenterprise import Key
from .types.recaptchaenterprise import ListKeysRequest
from .types.recaptchaenterprise import ListKeysResponse
from .types.recaptchaenterprise import RiskAnalysis
from .types.recaptchaenterprise import TokenProperties
from .types.recaptchaenterprise import UpdateKeyRequest
from .types.recaptchaenterprise import WebKeySettings


__all__ = (
    "AndroidKeySettings",
    "AnnotateAssessmentRequest",
    "AnnotateAssessmentResponse",
    "Assessment",
    "CreateAssessmentRequest",
    "CreateKeyRequest",
    "DeleteKeyRequest",
    "Event",
    "GetKeyRequest",
    "IOSKeySettings",
    "Key",
    "ListKeysRequest",
    "ListKeysResponse",
    "RiskAnalysis",
    "TokenProperties",
    "UpdateKeyRequest",
    "WebKeySettings",
    "RecaptchaEnterpriseServiceClient",
)



# -*- coding: utf-8 -*-

# Copyright 2020 Google LLC
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
#

from unittest import mock

import grpc
import math
import pytest

from google import auth
from google.api_core import client_options
from google.auth import credentials
from google.cloud.recaptchaenterprise_v1.services.recaptcha_enterprise_service import (
    RecaptchaEnterpriseServiceClient,
)
from google.cloud.recaptchaenterprise_v1.services.recaptcha_enterprise_service import (
    pagers,
)
from google.cloud.recaptchaenterprise_v1.services.recaptcha_enterprise_service import (
    transports,
)
from google.cloud.recaptchaenterprise_v1.types import recaptchaenterprise
from google.oauth2 import service_account
from google.protobuf import field_mask_pb2 as field_mask  # type: ignore
from google.protobuf import timestamp_pb2 as timestamp  # type: ignore


def test_recaptcha_enterprise_service_client_from_service_account_file():
    creds = credentials.AnonymousCredentials()
    with mock.patch.object(
        service_account.Credentials, "from_service_account_file"
    ) as factory:
        factory.return_value = creds
        client = RecaptchaEnterpriseServiceClient.from_service_account_file(
            "dummy/file/path.json"
        )
        assert client._transport._credentials == creds

        client = RecaptchaEnterpriseServiceClient.from_service_account_json(
            "dummy/file/path.json"
        )
        assert client._transport._credentials == creds

        assert client._transport._host == "recaptchaenterprise.googleapis.com:443"


def test_recaptcha_enterprise_service_client_client_options():
    # Check the default options have their expected values.
    assert (
        RecaptchaEnterpriseServiceClient.DEFAULT_OPTIONS.api_endpoint
        == "recaptchaenterprise.googleapis.com"
    )

    # Check that options can be customized.
    options = client_options.ClientOptions(api_endpoint="squid.clam.whelk")
    with mock.patch(
        "google.cloud.recaptchaenterprise_v1.services.recaptcha_enterprise_service.RecaptchaEnterpriseServiceClient.get_transport_class"
    ) as gtc:
        transport = gtc.return_value = mock.MagicMock()
        client = RecaptchaEnterpriseServiceClient(client_options=options)
        transport.assert_called_once_with(credentials=None, host="squid.clam.whelk")


def test_recaptcha_enterprise_service_client_client_options_from_dict():
    with mock.patch(
        "google.cloud.recaptchaenterprise_v1.services.recaptcha_enterprise_service.RecaptchaEnterpriseServiceClient.get_transport_class"
    ) as gtc:
        transport = gtc.return_value = mock.MagicMock()
        client = RecaptchaEnterpriseServiceClient(
            client_options={"api_endpoint": "squid.clam.whelk"}
        )
        transport.assert_called_once_with(credentials=None, host="squid.clam.whelk")


def test_create_assessment(transport: str = "grpc"):
    client = RecaptchaEnterpriseServiceClient(
        credentials=credentials.AnonymousCredentials(), transport=transport
    )

    # Everything is optional in proto3 as far as the runtime is concerned,
    # and we are mocking out the actual API, so just send an empty request.
    request = recaptchaenterprise.CreateAssessmentRequest()

    # Mock the actual call within the gRPC stub, and fake the request.
    with mock.patch.object(
        type(client._transport.create_assessment), "__call__"
    ) as call:
        # Designate an appropriate return value for the call.
        call.return_value = recaptchaenterprise.Assessment(name="name_value")

        response = client.create_assessment(request)

        # Establish that the underlying gRPC stub method was called.
        assert len(call.mock_calls) == 1
        _, args, _ = call.mock_calls[0]

        assert args[0] == request

    # Establish that the response is the type that we expect.
    assert isinstance(response, recaptchaenterprise.Assessment)
    assert response.name == "name_value"


def test_create_assessment_flattened():
    client = RecaptchaEnterpriseServiceClient(
        credentials=credentials.AnonymousCredentials()
    )

    # Mock the actual call within the gRPC stub, and fake the request.
    with mock.patch.object(
        type(client._transport.create_assessment), "__call__"
    ) as call:
        # Designate an appropriate return value for the call.
        call.return_value = recaptchaenterprise.Assessment()

        # Call the method with a truthy value for each flattened field,
        # using the keyword arguments to the method.
        response = client.create_assessment(
            parent="parent_value",
            assessment=recaptchaenterprise.Assessment(name="name_value"),
        )

        # Establish that the underlying call was made with the expected
        # request object values.
        assert len(call.mock_calls) == 1
        _, args, _ = call.mock_calls[0]
        assert args[0].parent == "parent_value"
        assert args[0].assessment == recaptchaenterprise.Assessment(name="name_value")


def test_create_assessment_flattened_error():
    client = RecaptchaEnterpriseServiceClient(
        credentials=credentials.AnonymousCredentials()
    )

    # Attempting to call a method with both a request object and flattened
    # fields is an error.
    with pytest.raises(ValueError):
        client.create_assessment(
            recaptchaenterprise.CreateAssessmentRequest(),
            parent="parent_value",
            assessment=recaptchaenterprise.Assessment(name="name_value"),
        )


def test_annotate_assessment(transport: str = "grpc"):
    client = RecaptchaEnterpriseServiceClient(
        credentials=credentials.AnonymousCredentials(), transport=transport
    )

    # Everything is optional in proto3 as far as the runtime is concerned,
    # and we are mocking out the actual API, so just send an empty request.
    request = recaptchaenterprise.AnnotateAssessmentRequest()

    # Mock the actual call within the gRPC stub, and fake the request.
    with mock.patch.object(
        type(client._transport.annotate_assessment), "__call__"
    ) as call:
        # Designate an appropriate return value for the call.
        call.return_value = recaptchaenterprise.AnnotateAssessmentResponse()

        response = client.annotate_assessment(request)

        # Establish that the underlying gRPC stub method was called.
        assert len(call.mock_calls) == 1
        _, args, _ = call.mock_calls[0]

        assert args[0] == request

    # Establish that the response is the type that we expect.
    assert isinstance(response, recaptchaenterprise.AnnotateAssessmentResponse)


def test_annotate_assessment_flattened():
    client = RecaptchaEnterpriseServiceClient(
        credentials=credentials.AnonymousCredentials()
    )

    # Mock the actual call within the gRPC stub, and fake the request.
    with mock.patch.object(
        type(client._transport.annotate_assessment), "__call__"
    ) as call:
        # Designate an appropriate return value for the call.
        call.return_value = recaptchaenterprise.AnnotateAssessmentResponse()

        # Call the method with a truthy value for each flattened field,
        # using the keyword arguments to the method.
        response = client.annotate_assessment(
            name="name_value",
            annotation=recaptchaenterprise.AnnotateAssessmentRequest.Annotation.LEGITIMATE,
        )

        # Establish that the underlying call was made with the expected
        # request object values.
        assert len(call.mock_calls) == 1
        _, args, _ = call.mock_calls[0]
        assert args[0].name == "name_value"
        assert (
            args[0].annotation
            == recaptchaenterprise.AnnotateAssessmentRequest.Annotation.LEGITIMATE
        )


def test_annotate_assessment_flattened_error():
    client = RecaptchaEnterpriseServiceClient(
        credentials=credentials.AnonymousCredentials()
    )

    # Attempting to call a method with both a request object and flattened
    # fields is an error.
    with pytest.raises(ValueError):
        client.annotate_assessment(
            recaptchaenterprise.AnnotateAssessmentRequest(),
            name="name_value",
            annotation=recaptchaenterprise.AnnotateAssessmentRequest.Annotation.LEGITIMATE,
        )


def test_create_key(transport: str = "grpc"):
    client = RecaptchaEnterpriseServiceClient(
        credentials=credentials.AnonymousCredentials(), transport=transport
    )

    # Everything is optional in proto3 as far as the runtime is concerned,
    # and we are mocking out the actual API, so just send an empty request.
    request = recaptchaenterprise.CreateKeyRequest()

    # Mock the actual call within the gRPC stub, and fake the request.
    with mock.patch.object(type(client._transport.create_key), "__call__") as call:
        # Designate an appropriate return value for the call.
        call.return_value = recaptchaenterprise.Key(
            name="name_value", display_name="display_name_value"
        )

        response = client.create_key(request)

        # Establish that the underlying gRPC stub method was called.
        assert len(call.mock_calls) == 1
        _, args, _ = call.mock_calls[0]

        assert args[0] == request

    # Establish that the response is the type that we expect.
    assert isinstance(response, recaptchaenterprise.Key)
    assert response.name == "name_value"
    assert response.display_name == "display_name_value"


def test_list_keys(transport: str = "grpc"):
    client = RecaptchaEnterpriseServiceClient(
        credentials=credentials.AnonymousCredentials(), transport=transport
    )

    # Everything is optional in proto3 as far as the runtime is concerned,
    # and we are mocking out the actual API, so just send an empty request.
    request = recaptchaenterprise.ListKeysRequest()

    # Mock the actual call within the gRPC stub, and fake the request.
    with mock.patch.object(type(client._transport.list_keys), "__call__") as call:
        # Designate an appropriate return value for the call.
        call.return_value = recaptchaenterprise.ListKeysResponse(
            next_page_token="next_page_token_value"
        )

        response = client.list_keys(request)

        # Establish that the underlying gRPC stub method was called.
        assert len(call.mock_calls) == 1
        _, args, _ = call.mock_calls[0]

        assert args[0] == request

    # Establish that the response is the type that we expect.
    assert isinstance(response, pagers.ListKeysPager)
    assert response.next_page_token == "next_page_token_value"


def test_list_keys_field_headers():
    client = RecaptchaEnterpriseServiceClient(
        credentials=credentials.AnonymousCredentials()
    )

    # Any value that is part of the HTTP/1.1 URI should be sent as
    # a field header. Set these to a non-empty value.
    request = recaptchaenterprise.ListKeysRequest(parent="parent/value")

    # Mock the actual call within the gRPC stub, and fake the request.
    with mock.patch.object(type(client._transport.list_keys), "__call__") as call:
        call.return_value = recaptchaenterprise.ListKeysResponse()
        client.list_keys(request)

        # Establish that the underlying gRPC stub method was called.
        assert len(call.mock_calls) == 1
        _, args, _ = call.mock_calls[0]
        assert args[0] == request

    # Establish that the field header was sent.
    _, _, kw = call.mock_calls[0]
    assert ("x-goog-request-params", "parent=parent/value") in kw["metadata"]


def test_list_keys_pager():
    client = RecaptchaEnterpriseServiceClient(
        credentials=credentials.AnonymousCredentials
    )

    # Mock the actual call within the gRPC stub, and fake the request.
    with mock.patch.object(type(client._transport.list_keys), "__call__") as call:
        # Set the response to a series of pages.
        call.side_effect = (
            recaptchaenterprise.ListKeysResponse(
                keys=[
                    recaptchaenterprise.Key(),
                    recaptchaenterprise.Key(),
                    recaptchaenterprise.Key(),
                ],
                next_page_token="abc",
            ),
            recaptchaenterprise.ListKeysResponse(keys=[], next_page_token="def"),
            recaptchaenterprise.ListKeysResponse(
                keys=[recaptchaenterprise.Key()], next_page_token="ghi"
            ),
            recaptchaenterprise.ListKeysResponse(
                keys=[recaptchaenterprise.Key(), recaptchaenterprise.Key()]
            ),
            RuntimeError,
        )
        results = [i for i in client.list_keys(request={})]
        assert len(results) == 6
        assert all(isinstance(i, recaptchaenterprise.Key) for i in results)


def test_list_keys_pages():
    client = RecaptchaEnterpriseServiceClient(
        credentials=credentials.AnonymousCredentials
    )

    # Mock the actual call within the gRPC stub, and fake the request.
    with mock.patch.object(type(client._transport.list_keys), "__call__") as call:
        # Set the response to a series of pages.
        call.side_effect = (
            recaptchaenterprise.ListKeysResponse(
                keys=[
                    recaptchaenterprise.Key(),
                    recaptchaenterprise.Key(),
                    recaptchaenterprise.Key(),
                ],
                next_page_token="abc",
            ),
            recaptchaenterprise.ListKeysResponse(keys=[], next_page_token="def"),
            recaptchaenterprise.ListKeysResponse(
                keys=[recaptchaenterprise.Key()], next_page_token="ghi"
            ),
            recaptchaenterprise.ListKeysResponse(
                keys=[recaptchaenterprise.Key(), recaptchaenterprise.Key()]
            ),
            RuntimeError,
        )
        pages = list(client.list_keys(request={}).pages)
        for page, token in zip(pages, ["abc", "def", "ghi", ""]):
            assert page.raw_page.next_page_token == token


def test_get_key(transport: str = "grpc"):
    client = RecaptchaEnterpriseServiceClient(
        credentials=credentials.AnonymousCredentials(), transport=transport
    )

    # Everything is optional in proto3 as far as the runtime is concerned,
    # and we are mocking out the actual API, so just send an empty request.
    request = recaptchaenterprise.GetKeyRequest()

    # Mock the actual call within the gRPC stub, and fake the request.
    with mock.patch.object(type(client._transport.get_key), "__call__") as call:
        # Designate an appropriate return value for the call.
        call.return_value = recaptchaenterprise.Key(
            name="name_value", display_name="display_name_value"
        )

        response = client.get_key(request)

        # Establish that the underlying gRPC stub method was called.
        assert len(call.mock_calls) == 1
        _, args, _ = call.mock_calls[0]

        assert args[0] == request

    # Establish that the response is the type that we expect.
    assert isinstance(response, recaptchaenterprise.Key)
    assert response.name == "name_value"
    assert response.display_name == "display_name_value"


def test_get_key_field_headers():
    client = RecaptchaEnterpriseServiceClient(
        credentials=credentials.AnonymousCredentials()
    )

    # Any value that is part of the HTTP/1.1 URI should be sent as
    # a field header. Set these to a non-empty value.
    request = recaptchaenterprise.GetKeyRequest(name="name/value")

    # Mock the actual call within the gRPC stub, and fake the request.
    with mock.patch.object(type(client._transport.get_key), "__call__") as call:
        call.return_value = recaptchaenterprise.Key()
        client.get_key(request)

        # Establish that the underlying gRPC stub method was called.
        assert len(call.mock_calls) == 1
        _, args, _ = call.mock_calls[0]
        assert args[0] == request

    # Establish that the field header was sent.
    _, _, kw = call.mock_calls[0]
    assert ("x-goog-request-params", "name=name/value") in kw["metadata"]


def test_update_key(transport: str = "grpc"):
    client = RecaptchaEnterpriseServiceClient(
        credentials=credentials.AnonymousCredentials(), transport=transport
    )

    # Everything is optional in proto3 as far as the runtime is concerned,
    # and we are mocking out the actual API, so just send an empty request.
    request = recaptchaenterprise.UpdateKeyRequest()

    # Mock the actual call within the gRPC stub, and fake the request.
    with mock.patch.object(type(client._transport.update_key), "__call__") as call:
        # Designate an appropriate return value for the call.
        call.return_value = recaptchaenterprise.Key(
            name="name_value", display_name="display_name_value"
        )

        response = client.update_key(request)

        # Establish that the underlying gRPC stub method was called.
        assert len(call.mock_calls) == 1
        _, args, _ = call.mock_calls[0]

        assert args[0] == request

    # Establish that the response is the type that we expect.
    assert isinstance(response, recaptchaenterprise.Key)
    assert response.name == "name_value"
    assert response.display_name == "display_name_value"


def test_delete_key(transport: str = "grpc"):
    client = RecaptchaEnterpriseServiceClient(
        credentials=credentials.AnonymousCredentials(), transport=transport
    )

    # Everything is optional in proto3 as far as the runtime is concerned,
    # and we are mocking out the actual API, so just send an empty request.
    request = recaptchaenterprise.DeleteKeyRequest()

    # Mock the actual call within the gRPC stub, and fake the request.
    with mock.patch.object(type(client._transport.delete_key), "__call__") as call:
        # Designate an appropriate return value for the call.
        call.return_value = None

        response = client.delete_key(request)

        # Establish that the underlying gRPC stub method was called.
        assert len(call.mock_calls) == 1
        _, args, _ = call.mock_calls[0]

        assert args[0] == request

    # Establish that the response is the type that we expect.
    assert response is None


def test_credentials_transport_error():
    # It is an error to provide credentials and a transport instance.
    transport = transports.RecaptchaEnterpriseServiceGrpcTransport(
        credentials=credentials.AnonymousCredentials()
    )
    with pytest.raises(ValueError):
        client = RecaptchaEnterpriseServiceClient(
            credentials=credentials.AnonymousCredentials(), transport=transport
        )


def test_transport_instance():
    # A client may be instantiated with a custom transport instance.
    transport = transports.RecaptchaEnterpriseServiceGrpcTransport(
        credentials=credentials.AnonymousCredentials()
    )
    client = RecaptchaEnterpriseServiceClient(transport=transport)
    assert client._transport is transport


def test_transport_grpc_default():
    # A client should use the gRPC transport by default.
    client = RecaptchaEnterpriseServiceClient(
        credentials=credentials.AnonymousCredentials()
    )
    assert isinstance(
        client._transport, transports.RecaptchaEnterpriseServiceGrpcTransport
    )


def test_recaptcha_enterprise_service_base_transport():
    # Instantiate the base transport.
    transport = transports.RecaptchaEnterpriseServiceTransport(
        credentials=credentials.AnonymousCredentials()
    )

    # Every method on the transport should just blindly
    # raise NotImplementedError.
    methods = (
        "create_assessment",
        "annotate_assessment",
        "create_key",
        "list_keys",
        "get_key",
        "update_key",
        "delete_key",
    )
    for method in methods:
        with pytest.raises(NotImplementedError):
            getattr(transport, method)(request=object())


def test_recaptcha_enterprise_service_auth_adc():
    # If no credentials are provided, we should use ADC credentials.
    with mock.patch.object(auth, "default") as adc:
        adc.return_value = (credentials.AnonymousCredentials(), None)
        RecaptchaEnterpriseServiceClient()
        adc.assert_called_once_with(
            scopes=("https://www.googleapis.com/auth/cloud-platform",)
        )


def test_recaptcha_enterprise_service_host_no_port():
    client = RecaptchaEnterpriseServiceClient(
        credentials=credentials.AnonymousCredentials(),
        client_options=client_options.ClientOptions(
            api_endpoint="recaptchaenterprise.googleapis.com"
        ),
        transport="grpc",
    )
    assert client._transport._host == "recaptchaenterprise.googleapis.com:443"


def test_recaptcha_enterprise_service_host_with_port():
    client = RecaptchaEnterpriseServiceClient(
        credentials=credentials.AnonymousCredentials(),
        client_options=client_options.ClientOptions(
            api_endpoint="recaptchaenterprise.googleapis.com:8000"
        ),
        transport="grpc",
    )
    assert client._transport._host == "recaptchaenterprise.googleapis.com:8000"


def test_recaptcha_enterprise_service_grpc_transport_channel():
    channel = grpc.insecure_channel("http://localhost/")
    transport = transports.RecaptchaEnterpriseServiceGrpcTransport(channel=channel)
    assert transport.grpc_channel is channel


def test_key_path():
    project = "squid"
    key = "clam"

    expected = "projects/{project}/keys/{key}".format(project=project, key=key)
    actual = RecaptchaEnterpriseServiceClient.key_path(project, key)
    assert expected == actual


def test_assessment_path():
    project = "squid"
    assessment = "clam"

    expected = "projects/{project}/assessments/{assessment}".format(
        project=project, assessment=assessment
    )
    actual = RecaptchaEnterpriseServiceClient.assessment_path(project, assessment)
    assert expected == actual
